@@ -1,7 +1,7 @@
 """
 pylibftdi - python wrapper for libftdi
 
-Copyright (c) 2010-2012 Ben Bass <benbass@codedstructure.net>
+Copyright (c) 2010-2013 Ben Bass <benbass@codedstructure.net>
 See LICENSE file for details and (absence of) warranty
 
 pylibftdi: http://bitbucket.org/codedstructure/pylibftdi
@@ -36,7 +36,8 @@ class BitBangDevice(Device):
                  lazy_open=False,
                  sync=True,
                  bitbang_mode=BITMODE_BITBANG,
-                 interface_select=None):
+                 interface_select=None,
+                 **kwargs):
         # initialise the super-class, but don't open yet. We really want
         # two-part initialisation here - set up all the instance variables
         # here in the super class, then open it after having set more
@@ -44,7 +45,8 @@ class BitBangDevice(Device):
         super(BitBangDevice, self).__init__(device_id=device_id,
                                             mode='b',
                                             lazy_open=True,
-                                            interface_select=interface_select)
+                                            interface_select=interface_select,
+                                            **kwargs)
         self.direction = direction
         self.sync = sync
         self.bitbang_mode = bitbang_mode
@@ -126,19 +128,24 @@ class BitBangDevice(Device):
         lines is persisted in this object for the purposes of reading,
         so read-modify-write operations (e.g. drv.port+=1) are valid.
         """
-        if self.sync:
-            result = self.read_pins()
+        if self._direction == ALL_OUTPUTS:
+            # a minor optimisation; no point reading from the port if
+            # we have no input lines set
+            result = self.latch
         else:
-            # the coercion to bytearray here is to make this work
-            # transparently between Python2 and Python3 - equivalent
-            # of ord() for Python2, a time-wasting do-nothing on Python3
-            result = bytearray(super(BitBangDevice, self).read(1))[0]
+            if self.sync:
+                result = self.read_pins()
+            else:
+                # the coercion to bytearray here is to make this work
+                # transparently between Python2 and Python3 - equivalent
+                # of ord() for Python2, a time-wasting do-nothing on Python3
+                result = bytearray(super(BitBangDevice, self).read(1))[0]
 
-        # replace the 'output' bits with current value of self.latch -
-        # the last written value. This makes read-modify-write
-        # operations (e.g. 'drv.port |= 0x10') work as expected
-        result = ((result & ~self._direction) |    # read input
-                  (self.latch & self._direction))  # output latch
+            # replace the 'output' bits with current value of self.latch -
+            # the last written value. This makes read-modify-write
+            # operations (e.g. 'drv.port |= 0x10') work as expected
+            result = ((result & ~self._direction) |    # read input
+                      (self.latch & self._direction))  # output latch
         return result
 
     @port.setter
