@@ -27,10 +27,10 @@ class ftdi_device_list(Structure):
 
 class ftdi_version_info(Structure):
     _fields_ = [('major', c_int),
-        ('minor', c_int),
-        ('micro', c_int),
-        ('version_str', c_char_p),
-        ('snapshot_str', c_char_p)]
+                ('minor', c_int),
+                ('micro', c_int),
+                ('version_str', c_char_p),
+                ('snapshot_str', c_char_p)]
 
 # Note I gave up on attempts to use ftdi_new/ftdi_free (just using
 # ctx instead of byref(ctx) in first param of most ftdi_* functions) as
@@ -49,12 +49,13 @@ FLUSH_OUTPUT = 3
 BITMODE_RESET = 0x00
 BITMODE_BITBANG = 0x01
 
-# Default USB IDs
-USB_VID = 0x0403
-USB_PID = 0x6001
-# However, a list of IDs is actually matched against.
-USB_VID_LIST = [USB_VID]
-USB_PID_LIST = [USB_PID, 0x6014]
+# Opening / searching for a device uses this list of IDs to search
+# by default. These can be extended directly after import if required.
+FTDI_VENDOR_ID = 0x0403
+USB_VID_LIST = [FTDI_VENDOR_ID]
+USB_PID_LIST = [0x6001, 0x6010, 0x6011, 0x6014]
+
+FTDI_ERROR_DEVICE_NOT_FOUND = -3
 
 
 class Driver(object):
@@ -76,8 +77,8 @@ class Driver(object):
         # int/void* which ctypes uses, but some need setting here
         self.fdll.ftdi_get_error_string.restype = c_char_p
         self.fdll.ftdi_usb_get_strings.argtypes = (
-                c_void_p, c_void_p,
-                c_char_p, c_int, c_char_p, c_int, c_char_p, c_int)
+            c_void_p, c_void_p,
+            c_char_p, c_int, c_char_p, c_int, c_char_p, c_int)
 
     def _find_libftdi(self, libftdi_search=None):
         """
@@ -105,10 +106,15 @@ class Driver(object):
         """
         return the version of the underlying library being used
         """
-        version = ftdi_version_info()
-        self.fdll.ftdi_get_library_version(byref(version))
-        return (version.major, version.minor, version.micro,
-                version.version_str, version.snapshot_str)
+        if hasattr(self.fdll, 'ftdi_get_library_version'):
+            version = ftdi_version_info()
+            self.fdll.ftdi_get_library_version(byref(version))
+            return (version.major, version.minor, version.micro,
+                    version.version_str, version.snapshot_str)
+        else:
+            # library versions <1.0 don't support this function...
+            return (0, 0, 0,
+                    'unknown - no ftdi_get_library_version()', 'unknown')
 
     def list_devices(self):
         """
@@ -140,24 +146,29 @@ class Driver(object):
         try:
             for usb_vid, usb_pid in itertools.product(USB_VID_LIST, USB_PID_LIST):
                 res = self.fdll.ftdi_usb_find_all(byref(ctx),
-                                                byref(dev_list_ptr),
-                                                usb_vid,
-                                                usb_pid)
+                                                  byref(dev_list_ptr),
+                                                  usb_vid,
+                                                  usb_pid)
                 if res < 0:
-                    raise FtdiError(self.fdll.ftdi_get_error_string(byref(ctx)))
+                    msg = "%s (%d)" % (self.get_error_string(), res)
+                    raise FtdiError(msg)
                 elif res > 0:
                     # take a copy of the dev_list for subsequent list_free
                     dev_list_base = pointer(dev_list_ptr.contents)
                     # traverse the linked list...
                     try:
                         while dev_list_ptr:
-                            res = self.fdll.ftdi_usb_get_strings(byref(ctx),
-                                    dev_list_ptr.contents.dev,
-                                    manuf, 127, desc, 127, serial, 127)
-                            if res < 0:
-                                raise FtdiError(self.fdll.ftdi_get_error_string(byref(ctx)))
+                            res = self.fdll.ftdi_usb_get_strings(
+                                byref(ctx),
+                                dev_list_ptr.contents.dev,
+                                manuf, 127, desc, 127, serial, 127)
+                            # don't error on failure to get all the data
+                            # error codes: -7: manuf, -8: desc, -9: serial
+                            if res < 0 and res not in (-7, -8, -9):
+                                msg = "%s (%d)" % (self.get_error_string(), res)
+                                raise FtdiError(msg)
                             devices.append((manuf.value, desc.value, serial.value))
-                            # step to next in linked-list if not
+                            # step to next in linked-list
                             dev_list_ptr = cast(dev_list_ptr.contents.next,
                                                 devlistptrtype)
                     finally:
@@ -257,7 +268,7 @@ class Device(object):
 
         if self.interface_select is not None:
             res = self.fdll.ftdi_set_interface(byref(self.ctx),
-                self.interface_select)
+                                               self.interface_select)
             if res != 0:
                 msg = "%s (%d)" % (self.get_error_string(), res)
                 del self.ctx
@@ -280,7 +291,9 @@ class Device(object):
                     #  - attempt to match device_id to description
                     open_args[-2], open_args[-1] = open_args[-1], open_args[-2]
                     res = self.fdll.ftdi_usb_open_desc(*tuple(open_args))
-            if res == 0:
+            if res != FTDI_ERROR_DEVICE_NOT_FOUND:
+                # if we succeed (0) or get a specific error, don't continue
+                # otherwise (-3) - look for another device
                 break
 
         if res != 0:
@@ -437,7 +450,7 @@ class Device(object):
             fn = self.fdll.ftdi_usb_purge_tx_buffer
         else:
             raise ValueError("Invalid value passed to %s.flush()" %
-                    self.__class__.__name__)
+                             self.__class__.__name__)
         res = fn(byref(self.ctx))
         if res != 0:
             msg = "%s (%d)" % (self.get_error_string(), res)
@@ -471,14 +484,15 @@ class Device(object):
         code (and import byref from ctypes):
 
         >>> with Device() as dev:
-        >>>     # set 8 bit data, 2 stop bits, no parity
-        >>>     dev.ftdi_fn.ftdi_set_line_property(8, 2, 0)
-        >>>     ...
+        ...     # set 8 bit data, 2 stop bits, no parity
+        ...     dev.ftdi_fn.ftdi_set_line_property(8, 2, 0)
+        ...
         """
         # note this class is constructed on each call, so this
         # won't be particularly quick.  It does ensure that the
         # fdll and ctx objects in the closure are up-to-date, though.
         class FtdiForwarder(object):
+
             def __getattr__(innerself, key):
                 return functools.partial(getattr(self.fdll, key),
                                          byref(self.ctx))
@@ -492,8 +506,8 @@ class Device(object):
         when used in a with statement, and the device object
         itself is returned:
         >>> with Device(mode='t') as dev:
-        >>>     dev.write('Hello World!')
-        >>>
+        ...     dev.write('Hello World!')
+        ...
         """
         self.open()
         return self
@@ -531,7 +545,7 @@ class Device(object):
                 break
             line_buffer.append(next_char)
             if (len(line_buffer) >= lsl and
-                line_buffer[-lsl:] == list(os.linesep)):
+                    line_buffer[-lsl:] == list(os.linesep)):
                 break
         return ''.join(line_buffer)
 
